<?php
namespace Teknisa\OPL\Modules\Consolidacao\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\OPL\Modules\Consolidacao\Services\CalculoNecessidadeOLService;

class CalculoNecessidadeOL extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $crudService;
    protected $dataSourceManager;
    protected $entityManager;
    
    public function __construct($crudService, $parameterBag, $dataSourceManager, EntityManager $entityManager, CalculoNecessidadeOLService $service){
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager = $entityManager;
        $this->service = $service;
	}
	
    protected function getCentralCompras(){
        return $this->crudService->getFilteredDataSet("sup07500_centralCompras", $this->service->criaFiltroCentral())->getRows();   
    }
    
    protected function getParametros($row){
        return $this->crudService->getFilteredDataSet("sup07500_parametros", $this->service->criaFiltroParametros($row))->getRows();   
    }
    
    public function setDadosCalculo(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $central = $this->getCentralCompras();
        $parametros = $this->getParametros($central);
        
        $resposta['IDUTILCONSSOLIOL']    = $parametros[0]['IDUTILCONSSOLIOL'];
        $resposta['CDGRUPPROD']          = $parametros[0]['CDGRUPPROD'];
        $resposta["IDGERACMPOLPER"]      = $parametros[0]["IDGERACMPOLPER"];
        $resposta["IDUTILCONSSOLIOLEXT"] = $parametros[0]["IDUTILCONSSOLIOLEXT"];
        
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    public function getSolicitacoes(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        $resposta = $this->crudService->getFilteredDataSet("sup07500_Solicitacoes", $conditions)->getRows();
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    public function verificaConsOL(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $this->service->VERIFICA_CONS_OL($row);
        
        $resposta = $this->getRelatIncons($row);
        
        $response->addDataSet(new DataSet("resposta", $resposta));    
    }
    
    protected function getRelatIncons($row){
        return $this->crudService->getFilteredDataSet("sup07600_getInsconsConsOL", $this->service->criaFiltroRelatInconsistencia($row))->getRows();   
    }
    
    protected function getFilialPevCons($row){
        return $this->crudService->getFilteredDataSet("sup07500_FilialPevCons", $this->service->criaFiltroFilialPrevCons($row))->getRows();
    }
    
    protected function getParamServico($row){
        $prevcons = $this->getFilialPevCons($row);
        $stFiliais = '_';
        for($i=0; $i<sizeof($prevcons); $i++) {
		    $stFiliais .= $prevcons[$i]['CDFILIAL'] . '_';
		}
		$resposta[0] = $stFiliais;
        $resposta[1] = $this->crudService->getFilteredDataSet("sup07500_ParamServico", $this->service->criaFiltroParamServico($stFiliais, $row['CDGRUPFILI']))->getRows();   
        return $resposta;
    }
    
    protected function getLimiSegu($cdfilial, $row){
        return $this->crudService->getFilteredDataSet("sup07500_getLimiSegu", $this->service->criaFiltroLimiSegu($cdfilial, $row))->getRows();   
    }
    
    public function previsaoConsumo(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $paramservico = $this->getParamServico($row);
        if(sizeof($paramservico[1]) == 0){
            $limite = $this->getLimiSegu($paramservico[0], $row);
            $dtfinallimi = $this->service->dtFinalLimiteSeg($row['DTFIM'], $limite[0]['NRLIMISEGU']);
            try {
                $this->service->CALCULA_PREVCONS_FILIAIS($row, $paramservico[0], $dtfinallimi);
            }
            catch(\Exception $e){
                $resposta['MSG'] = $e->getMessage();
                $response->addDataSet(new DataSet("resposta", $resposta));
            }
            $resposta['MSG'] = null;
        } else {
            $resposta['MSG'] = "ERROR";
        }
        $response->addDataSet(new DataSet("resposta", $resposta));
    }
    
    public function excluiInconsistencias(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $nrsequconsol = $row[0]["NRSEQUCONSOL"];
        $this->service->DelIncosistencia($nrsequconsol);
        for($i=0;$i<sizeof($row);$i++){
            if($nrsequconsol !== $row[$i]["NRSEQUCONSOL"]){
                $nrsequconsol = $row[$i]["NRSEQUCONSOL"];
                $this->service->DelIncosistencia($nrsequconsol);   
            }
        }
    }
    
    protected function getConfereNeceComp($codigo) {
        return $this->crudService->getFilteredDataSet("sup07500_getConfereNeceComp", $this->service->criaFiltroConfereNeceComp($codigo))->getRows();   
    }
    
    public function GetPrxNeceComp(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        try {
            $resposta[0] = $this->service->NECESSIDADE_OL($row);
            
            if(substr($resposta[0], -1) == 'S') {
                if($row['consPrevisaoConsumo'] == 'N') {
                    $vdnececompra = $this->service->getVdNeceCompra($row);
                    
                    if($vdnececompra[0]["CONTADOR"] > 0) {
                        $resposta[0] = $this->service->NECESSIDADE_OL($row);
                    }
                }
            }
            
            if($row['consPrevisaoConsumo'] == 'S') {
                $confereNeceComp = $this->getConfereNeceComp(substr($resposta[0], 0, 10));
                $resposta[2] = $confereNeceComp;
            } else {
                $resposta[2] = null;
            }
            
            $resposta[1] = null;
            $response->addDataSet(new DataSet("resposta", $resposta));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deletaTrabNece(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $prevcons = '';
        for($i=0; $i<sizeof($row); $i++) {
            if($prevcons !== $row[$i]['PREVCONS']){
                $prevcons = $row[$i]['PREVCONS'];    
    
                $params = array(
            		"P_TRABNECECOMP" => $prevcons,
            		"NRORG" => $this->environment->getNrorg()
            	);
            	$sql = "DELETE FROM TRABNECECOMP WHERE NRNECECOMP = :P_TRABNECECOMP AND NRORG = :NRORG";
                $stmt = $this->entityManager->getConnection()->prepare($sql);
                $stmt->execute($params);
            }
        }
    }
}       